@@ -9,7 +9,6 @@ from Products.Archetypes.utils import shasattr
 from Products.ATContentTypes.content.base import registerATCT
 from Products.ATContentTypes.content.base import ATCTContent
 from Products.ATContentTypes.content.schemata import ATContentTypeSchema
-from Products.ATContentTypes.content.schemata import finalizeATCTSchema
 from Products.ATContentTypes.configuration import zconf
 
 from Products.TALESField import TALESString, TALESLines
@@ -17,6 +16,7 @@ from Products.TALESField import TALESString, TALESLines
 from Products.PloneFormGen.config import PROJECTNAME, EDIT_TALES_PERMISSION
 
 from Products.CMFCore.permissions import View, ModifyPortalContent
+from Products.CMFCore.utils import getToolByName
 
 from AccessControl import ClassSecurityInfo
 
@@ -30,12 +30,16 @@ from Products.PloneFormGen.content.fieldsBase import *
 #    BaseFormField, BaseFieldSchemaStringDefault, \
 #    maxlengthField, sizeField, vocabularyField, vocabularyOverrideField
 
+from types import StringTypes, BooleanType
+from DateTime import DateTime
+import cgi
+
 class FGStringField(BaseFormField):
     """ A string entry field """
 
     security  = ClassSecurityInfo()
 
-    schema = BaseFieldSchemaStringDefault + Schema((
+    schema = BaseFieldSchemaStringDefault.copy() + Schema((
         maxlengthField,
         sizeField,
         StringField('fgStringValidator',
@@ -51,7 +55,7 @@ class FGStringField(BaseFormField):
     ))
 
     # hide references & discussion
-    finalizeATCTSchema(schema, folderish=True, moveDiscussion=False)
+    finalizeFieldSchema(schema, folderish=True, moveDiscussion=False)
 
     # Standard content type setup
     portal_type = meta_type = 'FormStringField'
@@ -72,74 +76,30 @@ class FGStringField(BaseFormField):
             validators=('isNotTooLong',),
             )
 
+
     def stringValidatorsDL(self):
         """ return a display list of string validators.
-            this is a hack for 118n
         """
 
-        # Check for Plone 2.5
-        if HAS_PLONE25:
-          return DisplayList( (
-                ('',
-                    _(u'vocabulary_none_text', u'None')
-                    ),
-                ('isEmail',
-                    _(u'vocabulary_isemailaddress_text', u'Is an E-Mail Address')
-                    ),
-                ('isPrintable',
-                    _(u'vocabulary_onlyprintable_text', u'Contains only printable characters')
-                    ),
-                ('isURL',
-                    _(u'vocabulary_isurl_text', u'Is a well-formed URL')
-                    ),
-                ('isUSPhoneNumber',
-                    _(u'vocabulary_isusphone_text', u'Is a valid US phone number')
-                    ),
-                ('isInternationalPhoneNumber',
-                    _(u'vocabulary_isintphone_text', u'Is a valid international phone number')
-                    ),
-                ('isZipCode',
-                    _(u'vocabulary_iszipcode_text', u'Is a valid zip code')
-                    ),
-                ) )
+        fgt = getToolByName(self, 'formgen_tool')
+        return fgt.getStringValidatorsDL()
+
+
+    def setFgStringValidator(self, value, **kw):
+        """ set simple validator """
+
+        fgt = getToolByName(self, 'formgen_tool')
+
+        if value and (value != 'vocabulary_none_text'):
+            fgtid = fgt.stringValidators[value].get('id')
+            if fgtid:
+                self.fgField.validators = ('isNotTooLong', fgtid)
         else:
-            return DisplayList( (
-                ('',
-                    self.translate( msgid='vocabulary_none_text',
-                    domain='ploneformgen',
-                    default='None')
-                    ),
-                ('isEmail',
-                    self.translate( msgid='vocabulary_isemailaddress_text',
-                    domain='ploneformgen',
-                    default='Is an E-Mail Address')
-                    ),
-                ('isPrintable',
-                    self.translate( msgid='vocabulary_onlyprintable_text',
-                    domain='ploneformgen',
-                    default='Contains only printable characters')
-                    ),
-                ('isURL',
-                    self.translate( msgid='vocabulary_isurl_text',
-                    domain='ploneformgen',
-                    default='Is a well-formed URL')
-                    ),
-                ('isUSPhoneNumber',
-                    self.translate( msgid='vocabulary_isusphone_text',
-                    domain='ploneformgen',
-                    default='Is a valid US phone number')
-                    ),
-                ('isInternationalPhoneNumber',
-                    self.translate( msgid='vocabulary_isintphone_text',
-                    domain='ploneformgen',
-                    default='Is a valid international phone number')
-                    ),
-                ('isZipCode',
-                    self.translate( msgid='vocabulary_iszipcode_text',
-                    domain='ploneformgen',
-                    default='Is a valid zip code')
-                    ),
-                ) )
+            self.fgField.validators = ('isNotTooLong',)
+        self.fgField._validationLayer()
+
+        self.fgStringValidator = value
+
 
 registerATCT(FGStringField, PROJECTNAME)
 
@@ -147,13 +107,17 @@ registerATCT(FGStringField, PROJECTNAME)
 class FGPasswordField(FGStringField):
     """ Password entry field (input is masked) """
 
-    schema = BaseFieldSchemaStringDefault + Schema((
+    schema = BaseFieldSchemaStringDefault.copy() + Schema((
         maxlengthField,
         sizeField,
     ))
 
+    # 'hidden' isn't really useful for this field.
+    del schema['hidden']
+
     # hide references & discussion
-    finalizeATCTSchema(schema, folderish=True, moveDiscussion=False)
+    finalizeFieldSchema(schema, folderish=True, moveDiscussion=False)
+
     # Standard content type setup
     portal_type = meta_type = 'FormPasswordField'
     archetype_name = 'Password Field'
@@ -182,7 +146,7 @@ class FGIntegerField(BaseFormField):
 
     security  = ClassSecurityInfo()
 
-    schema = BaseFieldSchemaStringDefault + Schema((
+    schema = BaseFieldSchemaStringDefault.copy() + Schema((
         IntegerField('minval',
             searchable=0,
             required=1,
@@ -214,8 +178,12 @@ class FGIntegerField(BaseFormField):
         maxlengthField,
         sizeField,
     ))
+
+    # 'hidden' isn't really useful for this field.
+    del schema['hidden']
+
     # hide references & discussion
-    finalizeATCTSchema(schema, folderish=True, moveDiscussion=False)
+    finalizeFieldSchema(schema, folderish=True, moveDiscussion=False)
 
     # Standard content type setup
     portal_type = meta_type = 'FormIntegerField'
@@ -246,10 +214,10 @@ class FGFixedPointField(BaseFormField):
 
     security  = ClassSecurityInfo()
 
-    schema = BaseFieldSchemaStringDefault + Schema((
+    schema = BaseFieldSchemaStringDefault.copy() + Schema((
         FixedPointField('minval',
             searchable=0,
-            required=1,
+            required=0,
             default='0.0',
             widget=DecimalWidget(
                 label="Minimum Acceptable Value",
@@ -280,8 +248,19 @@ class FGFixedPointField(BaseFormField):
         maxlengthField,
         sizeField,
     ))
+
+    # 'hidden' isn't really useful for this field.
+    del schema['hidden']
+
+    # and, required has only limited use ...
+    schema['required'].widget.description = \
+        """NOTE: For a fixed-point field, the required flag will not allow
+           entry of a '0' value.
+        """
+    schema['required'].widget.description_msgid = "help_fprequired_text"
+
     # hide references & discussion
-    finalizeATCTSchema(schema, folderish=True, moveDiscussion=False)
+    finalizeFieldSchema(schema, folderish=True, moveDiscussion=False)
 
     # Standard content type setup
     portal_type = meta_type = 'FormFixedPointField'
@@ -344,6 +323,30 @@ class FGBooleanField(BaseFormField):
                 description_msgid = "help_fgbooleanvalidator_text",
                 ),
         ),
+        StringField('fgBoolTrueString',
+            required=0,
+            searchable=0,
+            default='1',
+            widget=StringWidget(
+                label="True Display String",
+                description="""String to use in thanks page and mail when the field's checkbox is checked.""",
+                i18n_domain = "ploneformgen",
+                label_msgid = "label_fgbooleantruestring_text",
+                description_msgid = "help_fgbooleantruestring_text",
+                ),
+            ),
+        StringField('fgBoolFalseString',
+            required=0,
+            searchable=0,
+            default='0',
+            widget=StringWidget(
+                label="False Display String",
+                description="""String to use in thanks page and mail when the field's checkbox is unchecked.""",
+                i18n_domain = "ploneformgen",
+                label_msgid = "label_fgbooleanfalsestring_text",
+                description_msgid = "help_fgbooleanfalsestring_text",
+                ),
+            ),
     ))
     schema['required'].widget.description = \
         """NOTE: For a checkbox field, the required flag doesn't do anything beyond
@@ -351,8 +354,12 @@ class FGBooleanField(BaseFormField):
            particular input, choose a validator below.
         """
     schema['required'].widget.description_msgid = "help_boolrequired_text"
+
+    # 'hidden' isn't really useful for this field.
+    del schema['hidden']
+
     # hide references & discussion
-    finalizeATCTSchema(schema, folderish=True, moveDiscussion=False)
+    finalizeFieldSchema(schema, folderish=True, moveDiscussion=False)
 
     # Standard content type setup
     portal_type = meta_type = 'FormBooleanField'
@@ -420,6 +427,20 @@ class FGBooleanField(BaseFormField):
                 ) )
 
 
+    def htmlValue(self, REQUEST):
+        """ Return value instead of key """
+
+        value = REQUEST.form.get(self.__name__, 'No Input')
+        if type(value) == BooleanType:
+            if value:
+                return self.fgBoolTrueString
+        elif value == '1':
+            return self.fgBoolTrueString
+
+        return self.fgBoolFalseString
+
+
+
 registerATCT(FGBooleanField, PROJECTNAME)
 
 
@@ -428,7 +449,7 @@ class FGDateField(BaseFormField):
 
     security  = ClassSecurityInfo()
 
-    schema = BaseFieldSchemaStringDefault + Schema((
+    schema = BaseFieldSchemaStringDefault.copy() + Schema((
         BooleanField('fgShowHM',
             searchable=0,
             required=0,
@@ -440,9 +461,51 @@ class FGDateField(BaseFormField):
                 description_msgid = "help_fgshowhm_text",
                 ),
         ),
-    ))
+        IntegerField('fgStartingYear',
+            searchable=0,
+            required=0,
+            default='1999',
+            widget=IntegerWidget(
+                label='Starting Year',
+                i18n_domain = "ploneformgen",
+                label_msgid = "label_fgstartingyear_text",
+                description = "The first year to offer in the year drop-down.",
+                description_msgid = "help_fgstartingyear_text",
+                ),
+        ),
+        IntegerField('fgEndingYear',
+            searchable=0,
+            required=0,
+            default=None,
+            widget=IntegerWidget(
+                label='Ending Year',
+                i18n_domain = "ploneformgen",
+                label_msgid = "label_fgendingyear_text",
+                description = """The last year to offer in the year drop-down.
+                 Leave this empty if you wish to instead use a number of future years.""",
+                description_msgid = "help_fgendingyear_text",
+                ),
+        ),
+        IntegerField('fgFutureYears',
+            searchable=0,
+            required=0,
+            default='5',
+            widget=IntegerWidget(
+                label='Future Years To Display',
+                i18n_domain = "ploneformgen",
+                label_msgid = "label_fgfutureyears_text",
+                description = """The number of future years to offer in the year drop-down.
+                 (This is only applicable if you have not specified an ending year.)""",
+                description_msgid = "help_fgfutureyears_text",
+                ),
+        ),
+))
+
+    # 'hidden' isn't really useful for this field.
+    del schema['hidden']
+
     # hide references & discussion
-    finalizeATCTSchema(schema, folderish=True, moveDiscussion=False)
+    finalizeFieldSchema(schema, folderish=True, moveDiscussion=False)
 
     # Standard content type setup
     portal_type = meta_type = 'FormDateField'
@@ -463,13 +526,115 @@ class FGDateField(BaseFormField):
             widget=CalendarWidget(),
             )
 
+
     security.declareProtected(ModifyPortalContent, 'setFgShowHM')
     def setFgShowHM(self, value, **kw):
         """ set show_hm """
 
-        self.fgField.widget.show_hm = value == '1'
+        if type(value) == BooleanType:
+            self.fgField.widget.show_hm = value
+        else:
+            self.fgField.widget.show_hm = value == '1'
+
         self.fgShowHM = value
 
+
+    security.declareProtected(ModifyPortalContent, 'setFgStartingYear')
+    def setFgStartingYear(self, value, **kw):
+        """ set starting_year """
+
+        if value:
+            try:
+                self.fgField.widget.starting_year = int(value)
+                self.fgStartingYear = value
+            except ValueError:
+                pass
+        else:
+            self.fgField.widget.starting_year = None
+            self.fgStartingYear = value            
+            
+
+    security.declareProtected(ModifyPortalContent, 'setFgEndingYear')
+    def setFgEndingYear(self, value, **kw):
+        """ set ending_year """
+
+        if value:
+            try:
+                self.fgField.widget.ending_year = int(value)
+                self.fgEndingYear = value
+            except ValueError:
+                pass
+        else:
+                self.fgField.widget.ending_year = None
+                self.fgEndingYear = value
+
+
+    security.declareProtected(ModifyPortalContent, 'setFgFutureYears')
+    def setFgFutureYears(self, value, **kw):
+        """ set future_years """
+
+        if value:
+            try:
+                self.fgField.widget.future_years = int(value)
+                self.fgFutureYears = value
+            except ValueError:
+                pass
+        else:
+            self.fgField.widget.future_years = None
+            self.fgFutureYears = value            
+
+
+    def _toLocalizedTime(self, time, long_format=None):
+        tool = getToolByName(self, 'translation_service')
+        return tool.ulocalized_time(time, long_format=long_format)
+
+
+    def htmlValue(self, REQUEST):
+        """ return from REQUEST, this field's value, rendered as XHTML.
+        """
+
+        value = REQUEST.form.get(self.__name__, 'No Input')
+
+        # The replace('-','/') keeps the DateTime routine from
+        # interpreting this as UTC. Odd, but true.
+
+        try:
+            dt = DateTime(value.replace('-','/'))
+        except (DateTime.SyntaxError, DateTime.DateError):
+            # probably better to simply return the input
+            return cgi.escape(value)
+
+        if self.fgField.widget.show_hm:
+            ppt = getToolByName(self, 'portal_properties')
+            long_format = ppt.site_properties.localLongTimeFormat
+            value = self._toLocalizedTime(dt, long_format=long_format)
+        else:
+            value = self._toLocalizedTime(dt)
+
+        return cgi.escape(value)
+
+
+    def specialValidator(self, value, field, REQUEST, errors):
+        """ Archetypes isn't validating non-required dates --
+            so we need to.
+        """
+
+        fname = field.getName()
+        month = REQUEST.form.get('%s_month'%fname, '01')
+        day = REQUEST.form.get('%s_month'%fname, '01')
+        
+        if (month == '00') and (day == '00'):
+            value = ''
+            REQUEST.form[fname] = ''
+        
+        if value and not field.required:
+            try:
+                dt = DateTime(value)
+            except (DateTime.SyntaxError, DateTime.DateError):
+                return "Validation failed(isValidDate): this is not a valid date."
+        return 0        
+
+
 registerATCT(FGDateField, PROJECTNAME)
 
 
@@ -478,9 +643,9 @@ class FGLabelField(BaseFormField):
 
     security  = ClassSecurityInfo()
 
-    schema = BareFieldSchema
+    schema = BareFieldSchema.copy()
     # hide references & discussion
-    finalizeATCTSchema(schema, folderish=True, moveDiscussion=False)
+    finalizeFieldSchema(schema, folderish=True, moveDiscussion=False)
 
     # Standard content type setup
     portal_type = meta_type = 'FormLabelField'
@@ -501,6 +666,8 @@ class FGLabelField(BaseFormField):
             widget=LabelWidget(),
             )
 
+    def isLabel(self):
+        return True
 
 registerATCT(FGLabelField, PROJECTNAME)
 
@@ -510,7 +677,7 @@ class FGLinesField(BaseFormField):
 
     security  = ClassSecurityInfo()
 
-    schema = BaseFieldSchemaLinesDefault
+    schema = BaseFieldSchemaLinesDefault.copy()
 
     # Standard content type setup
     portal_type = meta_type = 'FormLinesField'
@@ -530,6 +697,13 @@ class FGLinesField(BaseFormField):
             write_permission = View,
             )
 
+    def fgPrimeDefaults(self, request, contextObject=None):
+        """ primes request with default """
+
+        # the lines widget will choke if there is no value
+        BaseFormField.fgPrimeDefaults(self, request, contextObject)
+        request.form.setdefault(self.fgField.__name__, ['',])
+
 
 registerATCT(FGLinesField, PROJECTNAME)
 
@@ -539,7 +713,7 @@ class FGSelectionField(BaseFormField):
 
     security  = ClassSecurityInfo()
 
-    schema = BaseFieldSchemaStringDefault + Schema((
+    schema = BaseFieldSchemaStringDefault.copy() + Schema((
         vocabularyField,
         vocabularyOverrideField,
         StringField('fgFormat',
@@ -556,8 +730,13 @@ class FGSelectionField(BaseFormField):
                 ),
         ),
     ))
+
+    # 'hidden' isn't really useful for a selection field.
+    # Just use a hidden string field if you really need this.
+    del schema['hidden']
+
     # hide references & discussion
-    finalizeATCTSchema(schema, folderish=True, moveDiscussion=False)
+    finalizeFieldSchema(schema, folderish=True, moveDiscussion=False)
 
     # Standard content type setup
     portal_type = meta_type = 'FormSelectionField'
@@ -625,6 +804,23 @@ class FGSelectionField(BaseFormField):
             ) )
 
 
+    def htmlValue(self, REQUEST):
+        """ Return value instead of key """
+
+        utils = getToolByName(self, 'plone_utils')
+        charset = utils.getSiteEncoding()
+
+        value = REQUEST.form.get(self.__name__, '')
+
+        # note that vocabulary items are in unicode;
+        # so, we must decode before lookup
+        vu = value.decode(charset)
+
+        vocabulary = self.fgField.Vocabulary(self)
+        v = vocabulary.getValue(vu) or vu
+
+        return cgi.escape(v.encode(charset))
+
 
 registerATCT(FGSelectionField, PROJECTNAME)
 
@@ -634,7 +830,7 @@ class FGMultiSelectField(BaseFormField):
 
     security  = ClassSecurityInfo()
 
-    schema = BaseFieldSchemaLinesDefault + Schema((
+    schema = BaseFieldSchemaLinesDefault.copy() + Schema((
         vocabularyField,
         vocabularyOverrideField,
         StringField('fgFormat',
@@ -651,8 +847,13 @@ class FGMultiSelectField(BaseFormField):
                 ),
         ),
     ))
+
+    # current Archetypes doesn't really support hidden for
+    # multi-select. Use a lines field if you really need this
+    del schema['hidden']
+
     # hide references & discussion
-    finalizeATCTSchema(schema, folderish=True, moveDiscussion=False)
+    finalizeFieldSchema(schema, folderish=True, moveDiscussion=False)
 
     # Standard content type setup
     portal_type = meta_type = 'FormMultiSelectionField'
@@ -726,6 +927,30 @@ class FGMultiSelectField(BaseFormField):
                 ) )
 
 
+    def htmlValue(self, REQUEST):
+        """ Return value instead of key """
+
+        utils = getToolByName(self, 'plone_utils')
+        charset = utils.getSiteEncoding()
+
+        value = REQUEST.form.get(self.__name__, [])
+
+        vocabulary = self.fgField.Vocabulary(self)
+        result = []
+        for k in value:
+            # there'll be an empty string to avoid
+            if len(k):
+                # vocabulary items are in unicode;
+                # so decode the key before lookup
+                ku = k.decode(charset)
+                v = vocabulary.getValue(ku) or ku
+                result.append(v)
+
+        value = u', '.join(result)
+
+        return cgi.escape(value.encode(charset))
+
+
 registerATCT(FGMultiSelectField, PROJECTNAME)
 
 
@@ -734,6 +959,11 @@ class PlainTextField(TextField):
     A text field forced text/plain.
     Without this hack, the textarea widget will (by acquisition) call
     the form folder's getContentType method.
+    
+    Also, the getAllowedContentTypes method override solves a problem
+    in Plone 3 where old instances of a text field with None set for
+    allowed_content_types were badly handled by the new getAllowedContentTypes
+    method in TextField -- which used site mimetype defaults.
     """
 
     security  = ClassSecurityInfo()
@@ -742,13 +972,17 @@ class PlainTextField(TextField):
     def getContentType(self, instance, fromBaseUnit=True):
         return 'text/plain'
 
+    security.declarePublic('getAllowedContentTypes')
+    def getAllowedContentTypes(self, instance):
+        return 'text/plain'        
+
 
 class FGTextField(BaseFormField):
     """ Text (textarea) field """
 
     security  = ClassSecurityInfo()
 
-    schema = BaseFieldSchemaTextDefault
+    schema = BaseFieldSchemaTextDefault.copy()
 
     # Standard content type setup
     portal_type = meta_type = 'FormTextField'
@@ -767,6 +1001,8 @@ class FGTextField(BaseFormField):
             required=0,
             write_permission = View,
             validators=('isNotTooLong',),
+            default_content_type = 'text/plain',
+            allowable_content_types = ('text/plain',),
             widget = TextAreaWidget(
                 maxlength=0,
                 ),
@@ -779,7 +1015,7 @@ class FGTextField(BaseFormField):
     security.declareProtected(View, 'getContentType')
     def getContentType(self, key=None):
         return 'text/plain'
-
+        
 
 registerATCT(FGTextField, PROJECTNAME)
 
@@ -801,12 +1037,16 @@ class HtmlTextField(TextField):
 class FGRichTextField(BaseFormField):
     """ Rich-text (visual editor) field """
 
-    from Products.ATContentTypes.config import HAS_MX_TIDY    
-    
+    from Products.ATContentTypes.config import HAS_MX_TIDY
+
     security  = ClassSecurityInfo()
 
     schema = BaseFieldSchemaRichTextDefault.copy()
 
+    # 'hidden' isn't really useful for an RT field.
+    # Just use a hidden string field if you really need this.
+    del schema['hidden']
+
     if HAS_MX_TIDY:
         schema = schema + Schema((
             StringField('fgStringValidator',
@@ -893,6 +1133,13 @@ class FGRichTextField(BaseFormField):
                     ),
                 ) )
 
+    def htmlValue(self, REQUEST):
+        """ override, as this field is already html.
+        """
+
+        return REQUEST.form.get(self.__name__, 'No Input')
+
+
 registerATCT(FGRichTextField, PROJECTNAME)
 
 
@@ -919,6 +1166,27 @@ class FGRichLabelField(BaseFormField):
                 allow_file_upload = False,
                 ),
             ),
+        TALESString('fgTDefault',
+            schemata='overrides',
+            searchable=0,
+            required=0,
+            validators=('talesvalidator',),
+            default='',
+            write_permission=EDIT_TALES_PERMISSION,
+            widget=StringWidget(label="Default Expression",
+                description="""
+                    A TALES expression that will be evaluated when the form is displayed
+                    to get the field default value.
+                    Leave empty if unneeded. Your expression should evaluate as a string.
+                    PLEASE NOTE: errors in the evaluation of this expression will cause
+                    an error on form display.
+                """,
+                size=70,
+                i18n_domain = "ploneformgen",
+                label_msgid = "label_fgtdefault_text",
+                description_msgid = "help_fgtdefault_text",
+                ),
+            ),
         ))
 
     schema['title'].widget.label = "Title"
@@ -968,6 +1236,9 @@ class FGRichLabelField(BaseFormField):
     def getContentType(self, key=None):
         return 'text/html'
 
+    def isLabel(self):
+        return True
+
 registerATCT(FGRichLabelField, PROJECTNAME)
 
 
@@ -991,7 +1262,11 @@ class FGFileField(BaseFormField):
         ),
 
     ))
-    finalizeATCTSchema(schema, folderish=True, moveDiscussion=False)
+
+    # 'hidden' isn't really useful for a file field.
+    del schema['hidden']
+
+    finalizeFieldSchema(schema, folderish=True, moveDiscussion=False)
 
     # Standard content type setup
     portal_type = meta_type = 'FormFileField'
@@ -1014,6 +1289,9 @@ class FGFileField(BaseFormField):
             validators=('isMaxSize',),
             )
 
+    def isFileField(self):
+        return True
+
     security.declareProtected(ModifyPortalContent, 'setFgMaxMB')
     def setFgMaxMB(self, value, **kw):
         """ set the maxmb """
@@ -1026,5 +1304,25 @@ class FGFileField(BaseFormField):
 
         return self.fgField.maxsize
 
+    def htmlValue(self, REQUEST):
+
+        from ZPublisher.HTTPRequest import FileUpload
+        from OFS.content_types import guess_content_type
+
+        file = REQUEST.form.get('%s_file' % self.fgField.getName())
+        if isinstance(file, FileUpload) and file.filename != '':
+            file.seek(0)
+            fdata = file.read()
+            filename = file.filename
+            mimetype, enc = guess_content_type(filename, fdata, None)
+            return "%s: %s bytes" % (mimetype, len(fdata))
+        else:
+            return 'No Input'
+
+    def getFieldFormName(self):
+        """field widget appends '_file' file to field id"""
+
+        return self.fgField.getName() + '_file'
+
 
 registerATCT(FGFileField, PROJECTNAME)
