@@ -7,15 +7,20 @@ from zope.interface import implements, providedBy
 
 import logging
 
-from AccessControl import ClassSecurityInfo
+from AccessControl import ClassSecurityInfo, Unauthorized
 from Products.CMFCore.permissions import View, ModifyPortalContent
 from Products.CMFCore.utils import getToolByName
 from Products.CMFCore.Expression import getExprContext
 
 from Products.CMFPlone.utils import safe_hasattr
 
-from Products.Archetypes.public import *
-from Products.Archetypes.utils import shasattr
+try:
+    from Products.LinguaPlone.public import *
+except ImportError:
+    from Products.Archetypes.public import *
+
+from Products.Archetypes.utils import shasattr, getRelURL
+from Products.Archetypes.interfaces.field import IField
 
 from Products.ATContentTypes.content.folder import ATFolderSchema, ATFolder
 from Products.ATContentTypes.content.schemata import finalizeATCTSchema
@@ -27,10 +32,14 @@ from Products.TALESField import TALESString
 from Products.PloneFormGen.interfaces import \
     IPloneFormGenForm, IPloneFormGenActionAdapter, IPloneFormGenThanksPage
 from Products.PloneFormGen.config import \
-    PROJECTNAME, fieldTypes, adapterTypes, thanksTypes, EDIT_TALES_PERMISSION
+    PROJECTNAME, fieldTypes, adapterTypes, thanksTypes, fieldsetTypes, \
+    EDIT_TALES_PERMISSION, EDIT_ADVANCED_PERMISSION, BAD_IDS
+from Products.PloneFormGen.content import validationMessages
 
 from Products.PloneFormGen import PloneFormGenMessageFactory as _
-from Products.PloneFormGen import HAS_PLONE25
+from Products.PloneFormGen import HAS_PLONE25, HAS_PLONE30
+
+from types import StringTypes
 
 if HAS_PLONE25:
   import zope.i18n
@@ -52,13 +61,24 @@ FormFolderSchema = ATFolderSchema.copy() + Schema((
     BooleanField('useCancelButton',
         required=0,
         searchable=0,
-        default='1',
-        widget=BooleanWidget(label='Show Cancel Button',
+        default='0',
+        languageIndependent=1,
+        widget=BooleanWidget(label='Show Reset Button',
             label_msgid = "label_showcancel_text",
             description_msgid = "help_showcancel_text",
             i18n_domain = "ploneformgen",
             ),
         ),
+    StringField('resetLabel',
+        required=0,
+        searchable=0,
+        default="Reset",
+        widget=StringWidget(
+                label="Reset Button Label",
+                label_msgid = "label_reset_button",
+                i18n_domain = 'ploneformgen',
+                ),
+        ),    
     LinesField('actionAdapter',
         vocabulary='actionAdaptersDL',
         widget=MultiSelectionWidget(
@@ -135,6 +155,7 @@ FormFolderSchema = ATFolderSchema.copy() + Schema((
         schemata='overrides',
         searchable=0,
         required=0,
+        languageIndependent=1,
         write_permission=EDIT_TALES_PERMISSION,
         widget=StringWidget(label="Custom Success Action",
             description="""
@@ -158,7 +179,9 @@ FormFolderSchema = ATFolderSchema.copy() + Schema((
         schemata='overrides',
         searchable=0,
         required=0,
+        write_permission=EDIT_ADVANCED_PERMISSION,
         validators='isURL',
+        languageIndependent=1,
         widget=StringWidget(label="Custom Form Action",
             description="""
                 Use this field to override the form action attribute.
@@ -179,6 +202,7 @@ FormFolderSchema = ATFolderSchema.copy() + Schema((
         validators=('talesvalidator',),
         write_permission=EDIT_TALES_PERMISSION,
         default='',
+        languageIndependent=1,
         widget=StringWidget(label="Form Setup Script",
             description="""
                 A TALES expression that will be called when the form is displayed.
@@ -202,6 +226,7 @@ FormFolderSchema = ATFolderSchema.copy() + Schema((
         validators=('talesvalidator',),
         write_permission=EDIT_TALES_PERMISSION,
         default='',
+        languageIndependent=1,
         widget=StringWidget(label="After Validation Script",
             description="""
                 A TALES expression that will be called after the form is 
@@ -228,6 +253,7 @@ FormFolderSchema = ATFolderSchema.copy() + Schema((
         validators=('talesvalidator',),
         write_permission=EDIT_TALES_PERMISSION,
         default='',
+        languageIndependent=1,
         widget=StringWidget(label="Header Injection",
             description="""
                 This override field allows you to insert content into the xhtml
@@ -245,6 +271,15 @@ FormFolderSchema = ATFolderSchema.copy() + Schema((
         ),
     ))
 
+if HAS_PLONE30:
+    # Plone 3 switches the schema tab widget to a select box when
+    # the number of schemata is > 6. IMHO, this has worse usability
+    # than packing the default schema.
+    # Also, as of P3.0, rich text fields on non-default schema
+    # still don't function.
+    FormFolderSchema['formPrologue'].schemata = 'default'
+    FormFolderSchema['formEpilogue'].schemata = 'default'
+
 #finalizeATCTSchema(ATFolderSchema, folderish=True, moveDiscussion=False)
 
 class FormFolder(ATFolder):
@@ -258,52 +293,132 @@ class FormFolder(ATFolder):
     meta_type      = 'FormFolder'
     portal_type    = 'FormFolder'
     archetype_name = 'Form Folder'
-    immediate_view = 'fg_base_view'
-    default_view   = 'fg_base_view'
+    if HAS_PLONE30:
+        default_view   = immediate_view = 'fg_base_view_p3'
+    else:
+        default_view   = immediate_view = 'fg_base_view'
     suppl_views = ()
 
     typeDescription= 'A folder which creates a form view from contained form fields.'
 
     # XXX We should do this with a tool so that others may add fields
-    allowed_content_types = fieldTypes + adapterTypes + thanksTypes + ('Document', 'Image')
+    allowed_content_types = fieldTypes + adapterTypes + thanksTypes + fieldsetTypes + ('Document', 'Image')
 
     security       = ClassSecurityInfo()
 
 
-    def _getFieldObjects(self, objTypes=None):
+    security.declarePrivate('_getFieldObjects')
+    def _getFieldObjects(self, objTypes=None, includeFSMarkers=False):
         """ return list of enclosed fields """
 
         # This function currently checks to see if
         # an object is a form field by looking to see
-        # if it has an fgField attribute. Alternatively,
-        # we could check to see if it provides IPloneFormGenField.
+        # if it has an fgField attribute.
+
+        # Make sure we look through fieldsets
+        if objTypes is not None:
+            objTypes = list(objTypes)[:]
+            objTypes.append('FieldsetFolder')
 
         myObjs = []
 
         for obj in self.objectValues(objTypes):
             # use shasattr to make sure we're not aquiring
-            # fgFields from the parent folder
-            if shasattr(obj, 'fgField'):
-                myObjs.append(obj)
+            # fgField by acquisition
+
+            # TODO: If I stick with this scheme for enable overrides,
+            # I'm probably going to want to find a way to cache the result
+            # in the request. _getFieldObjects potentially gets called
+            # several times in a request.
+
+            # first, see if the field enable override is set
+            if shasattr(obj, 'fgTEnabled') and obj.getRawFgTEnabled():
+                # process the override enabled TALES expression
+                # create a context for expression evaluation
+                context = getExprContext(self, obj)
+                # call the tales expression, passing our custom context
+                enabled = obj.getFgTEnabled(expression_context=context)
+            else:
+                enabled = True
+            
+            if enabled:
+                if shasattr(obj, 'fgField'):
+                    myObjs.append(obj)
+                if shasattr(obj, 'fieldsetFields'):
+                    myObjs += obj.fieldsetFields(objTypes, includeFSMarkers)
 
         return myObjs
 
 
+    security.declarePrivate('findFieldObjectByName')
+    def findFieldObjectByName(self, name):
+        """ Find a form field object by name,
+            searching fieldsets if necessary.
+            This is used by fieldsBase vocabulary fields
+            to find the form field instance associated
+            with a field.
+        """
+       
+        for obj in self._getFieldObjects():
+            if obj.__name__ == name:
+               return obj
+        return None        
+
+
+    security.declarePrivate('cleanExpressionContext')
+    def cleanExpressionContext(self, object=None, request=None):
+        """ clean the expression context of references to object """
+
+        # When a TALES expression is evaluated, a copy
+        # of the expression context for the object is stored in the request
+        # in case it's needed again.
+        # The problem is that if we allow this to get stored for PFG's
+        # overrides, it will not meet the needs of template code that
+        # assumes a more complete context.
+        #
+        # This function's job is to clear the expression context cache for
+        # the current object.
+        #
+        # Note that this doesn't need to be called if there's no danger
+        # of the expression context being re-used in a template.
+
+        if object is None:
+            object = self
+        
+        if request:
+            cache = request.get('_ec_cache', None)
+            if cache:
+                if cache.has_key( id(object) ):
+                    del cache[ id(object) ]
+        
+
     security.declareProtected(View, 'fgFields')
     def fgFields(self, request=None, displayOnly=False):
-        """ generate fields on the fly; also primes request with defaults if request is passed """
+        """ generate fields on the fly; also primes request with
+            defaults if request is passed.
+            if displayOnly, label fields are excluded.
+        """
 
         if request and self.getRawOnDisplayOverride():
             # call the tales expression, passing a custom context
-            self.getOnDisplayOverride(expression_context=getExprContext(self, self.aq_explicit))
-        
+            #self.getOnDisplayOverride(expression_context=getExprContext(self, self.aq_explicit))
+            self.getOnDisplayOverride()
+            self.cleanExpressionContext(request=request)
+
         myFields = []
-        for obj in self._getFieldObjects():
-            if request:
-                # prime the request
-                obj.fgPrimeDefaults(request)
-            if not (displayOnly and obj.meta_type in ('FormFileField','FormLabelField','FormRichLabelField')):
-                myFields.append(obj.fgField)
+        for obj in self._getFieldObjects(includeFSMarkers=not displayOnly):
+            if IField.isImplementedBy(obj):
+                # this is a field -- not a form field -- and may be
+                # added directly to the field list.
+                if not displayOnly:
+                    myFields.append(obj)
+            else:
+                if request:
+                    # prime the request
+                    obj.fgPrimeDefaults(request)
+                #if not (displayOnly and (obj.isLabel() or obj.isFileField()) ):
+                if not (displayOnly and obj.isLabel()):
+                    myFields.append(obj.fgField)
 
         return myFields
 
@@ -318,7 +433,8 @@ class FormFolder(ATFolder):
         if errors:
             return errors
 
-        fields = self._getFieldObjects()
+        # Get all the form fields. Exclude actual IField fields.
+        fields = [fo for fo in self._getFieldObjects() if not IField.isImplementedBy(fo)]
 
         for obj in fields:
             field = obj.fgField
@@ -331,21 +447,41 @@ class FormFolder(ATFolder):
             else:
                 value = result[0]
 
-            res = field.validate(instance=self,
-                                value=value,
-                                errors=errors,
-                                REQUEST=REQUEST)
+            # workaround what I consider a Zope marshalling error: the production
+            # of lists like ['one', ''] and [''] for list fields.
+            # no need to worry about polymorphism here, as this is a very particular
+            # case.        
+            if isinstance(value, type([])) and len(value) and \
+              (type(value[-1]) in StringTypes) and (len(value[-1]) == 0):
+                value.pop()
+                
+            # eliminate trailing white space in string types.
+            if safe_hasattr(value, 'rstrip'):
+                newvalue = value.rstrip()
+                if newvalue != value:
+                    value = newvalue
+                    # since strings are immutable, we have to manually store it back to the request
+                    if safe_hasattr(REQUEST, 'form'):
+                        REQUEST.form[obj.getFieldFormName()] = value                        
+
+            # Archetypes field validation
+            res = field.validate(instance=self, value=value, errors=errors, REQUEST=REQUEST)
+
+            if not res:
+                # give the field itself an opportunity to validate.
+                res = obj.specialValidator(value, field, REQUEST, errors)
 
             if res:
-                errors[field.getName()] = res
+                errors[field.getName()] = validationMessages.cleanupMessage(res, self.REQUEST, self)
             elif shasattr(obj, 'getFgTValidator') and obj.getRawFgTValidator():
+                # process the override validator TALES expression
 
                 # create a context for expression evaluation
                 context = getExprContext(self, obj)
                 
                 # put this field's input (from request) into the context globals
                 # as 'value'
-                context.global_vars['value'] = REQUEST.form.get(field.getName(), None)
+                context.setGlobal('value', REQUEST.form.get(obj.getFieldFormName(), None))
                 
                 # call the tales expression, passing our custom context
                 cerr = obj.getFgTValidator(expression_context=context)
@@ -354,16 +490,33 @@ class FormFolder(ATFolder):
 
         if not errors:
             if self.getRawAfterValidationOverride():
+                # evaluate the override.
+                # In case we end up traversing to a template,
+                # we need to make sure we don't clobber
+                # the expression context.
                 self.getAfterValidationOverride()
-                # self.getAfterValidationOverride(expression_context=getExprContext(self, self.aq_explicit))
+                self.cleanExpressionContext(request=self.REQUEST)
 
             adapters = self.getRawActionAdapter()
             for adapter in adapters:
-                try:
-                    actionAdapter = self[adapter]
-                    actionAdapter.onSuccess(fields, REQUEST=REQUEST)
-                except KeyError:
+                actionAdapter = getattr(self.aq_explicit, adapter, None)
+                if actionAdapter is None:
                     logger.warn("Designated action adapter '%s' is missing; ignored." % adapter)
+                else:
+                    # Now, see if we should execute it.
+                    # Check to see if execCondition exists and has contents
+                    if safe_hasattr(actionAdapter, 'execCondition') and \
+                      len(actionAdapter.getRawExecCondition()):
+                        # evaluate the execCondition.
+                        # create a context for expression evaluation
+                        context = getExprContext(self, actionAdapter)
+                        doit = actionAdapter.getExecCondition(expression_context=context)
+                    else:
+                        # no reason not to go ahead
+                        doit = True
+
+                    if doit:
+                        actionAdapter.onSuccess(fields, REQUEST=REQUEST)
 
         return errors
 
@@ -384,19 +537,12 @@ class FormFolder(ATFolder):
         if s:
             obj = getattr(self, s, None)
             if obj:
-                if IPloneFormGenThanksPage in providedBy(obj):
-                    # traverse to the object so that request is available.
-                    # An oddity: we can't just traverse to the ID, but seem
-                    # to need a path.
-                    path = '/'.join(obj.getPhysicalPath())
-                    return 'traverse_to:string:%s' % path
-                else:
-                    return 'redirect_to:string:%s' % s
+                return 'traverse_to:string:%s' % obj.getId()
             
         return 'traverse_to:string:fg_result_view'
 
 
-    # security.declareProtected(View, 'getRawActionAdapter')
+    # security.declareProtected(ModifyPortalContent, 'getRawActionAdapter')
     def getRawActionAdapter(self):
         """ Returns selected action adapters as tuple """
 
@@ -412,7 +558,7 @@ class FormFolder(ATFolder):
             return self.actionAdapter
 
 
-    security.declareProtected(View, 'actionAdaptersDL')
+    security.declareProtected(ModifyPortalContent, 'actionAdaptersDL')
     def actionAdaptersDL(self):
         """ returns Display List (id, title) tuples of contained adapters """
 
@@ -434,6 +580,17 @@ class FormFolder(ATFolder):
 #                    )
 
 
+    security.declareProtected(ModifyPortalContent, 'addActionAdapter')
+    def addActionAdapter(self, id):
+        """ activate action adapter with id == id """
+        
+        aa = list(self.getRawActionAdapter())
+        if id not in aa:
+            aa.append(id.decode(self.getCharset()))
+        self.actionAdapter = aa
+        
+
+    security.declareProtected(ModifyPortalContent, 'fgFieldsDisplayList')
     def fgFieldsDisplayList(self, withNone=False, objTypes=None):
         """ returns display list of fields """
 
@@ -453,6 +610,7 @@ class FormFolder(ATFolder):
         return DisplayList( myFields )
 
 
+    security.declareProtected(ModifyPortalContent, 'thanksPageVocabulary')
     def thanksPageVocabulary(self):
         """ returns a DisplayList of contained page-ish documents """
 
@@ -467,7 +625,7 @@ class FormFolder(ATFolder):
 
         for obj in self.objectValues():
             if IPloneFormGenThanksPage in providedBy(obj) or \
-             obj.portal_type in defaultPageTypes:
+              getattr(obj.aq_explicit, 'portal_type', 'none') in defaultPageTypes:
                 tpages.append( (obj.getId(), obj.title) )
                 
         return DisplayList( tpages )
@@ -485,7 +643,7 @@ class FormFolder(ATFolder):
 
         try:
             res = BaseObject.isBinary(self, key)
-        except TypeError:
+        except (TypeError, AttributeError):
             res = 0
         return res
 
@@ -503,6 +661,28 @@ class FormFolder(ATFolder):
         return False
 
 
+    security.declarePrivate('_pfFixup')
+    def _pfFixup(self, obj):
+    
+        # the creation of contained objects in initializeArchetypes
+        # leaves catalog orphans for the portal_factory objects.
+        #
+        # this solves the problem by removing them from the
+        # portal_catalog and uid_catalog,
+        # hopefully not causing other problems in the process.
+
+        if 'portal_factory' in obj.getPhysicalPath():
+            # remove from portal_catalog
+            obj.unindexObject()
+
+            # remove from uid_catalog
+            uid_catalog = getToolByName(self, 'uid_catalog', None)
+            uid_catalog.uncatalog_object( getRelURL(self, obj.getPhysicalPath()) )
+
+        else:
+            obj.reindexObject()
+        
+
     def initializeArchetype(self, **kwargs):
         """ Create sample content that may help folks
             figure out how this gadget works.
@@ -512,34 +692,45 @@ class FormFolder(ATFolder):
 
         if HAS_PLONE25:
             self.setSubmitLabel(zope.i18n.translate(_(u'pfg_formfolder_submit', u'Submit'), context=self.REQUEST))
+            self.setResetLabel(zope.i18n.translate(_(u'pfg_formfolder_reset', u'Reset'), context=self.REQUEST))
         else:
             self.setSubmitLabel(self.translate(
                                   msgid='pfg_formfolder_submit',
                                   domain='ploneformgen',
                                   default='Submit'))
+            self.setResetLabel(self.translate(
+                                  msgid='pfg_formfolder_reset',
+                                  domain='ploneformgen',
+                                  default='Reset'))
 
         oids = self.objectIds()
 
+        haveMailer = False
         if 'mailer' not in oids:
             # create a mail action
-            self.invokeFactory('FormMailerAdapter','mailer')
-            mailer = self['mailer']
+            try:
+                self.invokeFactory('FormMailerAdapter','mailer')
+                mailer = self['mailer']
 
-            if HAS_PLONE25:
-                mailer.setTitle(zope.i18n.translate(_(u'pfg_mailer_title', u'Mailer'), context=self.REQUEST))
-                mailer.setDescription(zope.i18n.translate(_(u'pfg_mailer_description', u'E-Mails Form Input'), context=self.REQUEST))
-            else:
-                mailer.setTitle(self.utranslate(
-                        msgid='pfg_mailer_title',
-                        domain='ploneformgen',
-                        default='Mailer'))
-                mailer.setDescription(self.utranslate(
-                        msgid='pfg_mailer_description',
-                        domain='ploneformgen',
-                        default='E-Mails From Input'))
+                if HAS_PLONE25:
+                    mailer.setTitle(zope.i18n.translate(_(u'pfg_mailer_title', u'Mailer'), context=self.REQUEST))
+                    mailer.setDescription(zope.i18n.translate(_(u'pfg_mailer_description', u'E-Mails Form Input'), context=self.REQUEST))
+                else:
+                    mailer.setTitle(self.utranslate(
+                            msgid='pfg_mailer_title',
+                            domain='ploneformgen',
+                            default='Mailer'))
+                    mailer.setDescription(self.utranslate(
+                            msgid='pfg_mailer_description',
+                            domain='ploneformgen',
+                            default='E-Mails From Input'))
 
-            mailer.reindexObject()
-            self.actionAdapter = ('mailer',)
+                self._pfFixup(mailer)
+
+                self.actionAdapter = ('mailer',)
+                haveMailer = True
+            except Unauthorized:
+                logger.warn('User not authorized to create mail adapters. Form Folder created with no action adapter.')
 
         if 'replyto' not in oids:
             # create a replyto field
@@ -558,9 +749,12 @@ class FormFolder(ATFolder):
             obj.fgField.required = True
             obj.setFgStringValidator('isEmail')
             obj.setFgTDefault('here/memberEmail')
-            obj.setFgDefault('dynamically overriden')
-            obj.reindexObject()
-            mailer.replyto_field = 'replyto'
+            obj.setFgDefault('dynamically overridden')
+
+            self._pfFixup(obj)
+
+            if haveMailer:
+                mailer.replyto_field = 'replyto'
         
         if 'topic' not in oids:
             # create a subject field
@@ -577,8 +771,11 @@ class FormFolder(ATFolder):
                                   default='Subject'))
 
             obj.fgField.required = True
-            obj.reindexObject()
-            mailer.subject_field = 'topic'
+
+            self._pfFixup(obj)
+
+            if haveMailer:
+                mailer.subject_field = 'topic'
         
         if 'comments' not in oids:
             # create a comments field
@@ -595,7 +792,9 @@ class FormFolder(ATFolder):
                                 default='Comments'))
 
             obj.fgField.required = True
-            obj.reindexObject()
+
+            self._pfFixup(obj)
+
         
         if 'thank-you' not in oids:
             # create a thanks page
@@ -615,7 +814,8 @@ class FormFolder(ATFolder):
                                     domain='ploneformgen',
                                     default='Thanks for your input.'))
 
-            obj.reindexObject()
+            self._pfFixup(obj)
+
             self.thanksPage = 'thank-you'
                     
 
@@ -660,6 +860,56 @@ class FormFolder(ATFolder):
         return member.id
 
 
+    # security is inherited
+    def checkIdAvailable(self, id):
+        """ Expands on ATFolder by checking for ids known to cause problems.
+            This includes ids of objects in all fieldsets.
+        """
+        
+        result = ATFolder.checkIdAvailable(self, id)
+        if result:
+            result = id not in BAD_IDS
+            if result:
+                # check the fieldsets
+                fieldsets = self.objectValues('FieldsetFolder')
+                for fs in fieldsets:
+                    if id in fs.objectIds():
+                        return False
+                
+        return result
+        
+
+    security.declareProtected(View, 'formFolderObject')
+    def formFolderObject(self):
+        """ Find form folder by acquisition """
+        
+        return self
+
+ 
+    security.declareProtected(ModifyPortalContent, 'setFormPrologue')
+    def setFormPrologue(self, value, **kw):
+        """ Set formPrologue """
+        
+        # workaround a Kupu oddity: saving '<p>&nbsp;</p>' for
+        # and empty input
+        if value.strip() == '<p>&nbsp;</p>':
+            self.formPrologue = ''
+        else:
+            self.formPrologue = value
+
+
+    security.declareProtected(ModifyPortalContent, 'setFormPrologue')
+    def setFormEpilogue(self, value, **kw):
+        """ Set formEpilogue """
+        
+        # workaround a Kupu oddity: saving '<p>&nbsp;</p>' for
+        # and empty input
+        if value.strip() == '<p>&nbsp;</p>':
+            self.formEpilogue = ''
+        else:
+            self.formEpilogue = value
+
+
 #    security.declareProtected(ModifyPortalContent, 'myi18n')
 #    def myi18n(self):
 #        """ return i18n declarations from widgets """
