@@ -35,7 +35,7 @@ from Products.PloneFormGen.content.actionAdapter import FormActionAdapter, FormA
 from Products.PageTemplates.ZopePageTemplate import ZopePageTemplate
 
 from Products.TALESField import TALESString, TALESLines
-from Products.TemplateFields import ZPTField
+from Products.TemplateFields import ZPTField as ZPTField
 
 from Products.PloneFormGen.content.actionAdapter import FormActionAdapter, FormAdapterSchema
 from ya_gpg import gpg
@@ -48,10 +48,23 @@ from email.MIMEImage import MIMEImage
 from email.MIMEMultipart import MIMEMultipart
 from email.MIMEText import MIMEText
 
+from types import StringTypes, StringType, UnicodeType 
+
 from DateTime import DateTime
 
 from Products.PloneFormGen import PloneFormGenMessageFactory as _
-from Products.PloneFormGen import HAS_PLONE25
+from Products.PloneFormGen import HAS_PLONE25, HAS_PLONE30
+
+try:
+    # 3.0+
+    from zope.contenttype import guess_content_type
+except ImportError:
+    try:
+        # 2.5
+        from zope.app.content_types import guess_content_type
+    except ImportError:
+        # 2.1
+        from OFS.content_types import guess_content_type
 
 if HAS_PLONE25:
   import zope.i18n
@@ -61,7 +74,8 @@ formMailerAdapterSchema = FormAdapterSchema.copy() + Schema((
     StringField('recipient_name',
         searchable=0,
         required=0,
-        schemata='addressing',
+        default_method='getDefaultRecipientName',
+        write_permission=EDIT_ADDRESSING_PERMISSION,
         read_permission=ModifyPortalContent,
         widget=StringWidget(
             label = "Recipient's full name",
@@ -76,7 +90,8 @@ formMailerAdapterSchema = FormAdapterSchema.copy() + Schema((
     StringField('recipient_email',
         searchable=0,
         required=0,
-        schemata='addressing',
+        default_method='getDefaultRecipient',
+        write_permission=EDIT_ADDRESSING_PERMISSION,
         read_permission=ModifyPortalContent,
         validators=('isEmail',),
         widget=StringWidget(
@@ -91,12 +106,13 @@ formMailerAdapterSchema = FormAdapterSchema.copy() + Schema((
         schemata='addressing',
         searchable=0,
         required=0,
+        write_permission=EDIT_ADVANCED_PERMISSION,
         read_permission=ModifyPortalContent,
         vocabulary='selectFieldsDisplayList',
         widget=SelectionWidget(
             label = 'Extract Recipient From',
             label_msgid = "label_formmailer_to_extract",
-            description = 
+            description =
                 """
                 Choose a form field from which you wish to extract
                 input for the To header. If you choose anything other
@@ -112,8 +128,9 @@ formMailerAdapterSchema = FormAdapterSchema.copy() + Schema((
     LinesField('cc_recipients',
         searchable=0,
         required=0,
-        default=[],
+        default_method='getDefaultCC',
         schemata='addressing',
+        write_permission=EDIT_ADDRESSING_PERMISSION,
         read_permission=ModifyPortalContent,
         widget=LinesWidget(
             label = 'CC Recipients',
@@ -127,7 +144,8 @@ formMailerAdapterSchema = FormAdapterSchema.copy() + Schema((
         schemata='addressing',
         searchable=0,
         required=0,
-        default=[],
+        default_method='getDefaultBCC',
+        write_permission=EDIT_ADDRESSING_PERMISSION,
         read_permission=ModifyPortalContent,
         widget=LinesWidget(
             label = 'BCC Recipients',
@@ -143,10 +161,11 @@ formMailerAdapterSchema = FormAdapterSchema.copy() + Schema((
         required=0,
         vocabulary='fieldsDisplayList',
         read_permission=ModifyPortalContent,
+        write_permission=EDIT_ADVANCED_PERMISSION,
         widget=SelectionWidget(
             label = 'Extract Reply-To From',
             label_msgid = "label_formmailer_replyto_extract",
-            description = 
+            description =
                 """
                 Choose a form field from which you wish to extract
                 input for the Reply-To header. NOTE: You should
@@ -164,7 +183,7 @@ formMailerAdapterSchema = FormAdapterSchema.copy() + Schema((
         default='Form Submission',
         read_permission=ModifyPortalContent,
         widget=StringWidget(
-            description = 
+            description =
                 """
                 Subject line of message. This is used if you
                 do not specify a subject field or if the field
@@ -181,11 +200,12 @@ formMailerAdapterSchema = FormAdapterSchema.copy() + Schema((
         searchable=0,
         required=0,
         vocabulary='fieldsDisplayList',
+        write_permission=EDIT_ADVANCED_PERMISSION,
         read_permission=ModifyPortalContent,
         widget=SelectionWidget(
             label = 'Extract Subject From',
             label_msgid = "label_formmailer_subject_extract",
-            description = 
+            description =
                 """
                 Choose a form field from which you wish to extract
                 input for the mail subject line.
@@ -235,7 +255,7 @@ formMailerAdapterSchema = FormAdapterSchema.copy() + Schema((
     ZPTField('body_pt',
         schemata='template',
         write_permission=EDIT_TALES_PERMISSION,
-        default = DEFAULT_MAILTEMPLATE_BODY,
+        default_method = 'getMailBodyDefault',
         read_permission=ModifyPortalContent,
         widget=TextAreaWidget(description = 'This is a Zope Page Template '
             'used for rendering of the mail-body. You don\'t need to modify '
@@ -252,8 +272,9 @@ formMailerAdapterSchema = FormAdapterSchema.copy() + Schema((
         ),
     StringField('body_type',
         schemata='template',
-        default='html',
+        default_method='getMailBodyTypeDefault',
         vocabulary = MIME_LIST,
+        write_permission=EDIT_ADVANCED_PERMISSION,
         read_permission=ModifyPortalContent,
         widget=SelectionWidget(description = 'Set the mime-type of the mail-body. '
             'Change this setting only if you know exactly what you are doing. '
@@ -268,7 +289,8 @@ formMailerAdapterSchema = FormAdapterSchema.copy() + Schema((
         searchable=0,
         required=0,
         schemata='headers',
-        default=('HTTP_X_FORWARDED_FOR', 'REMOTE_ADDR', 'PATH_INFO'), 
+        default_method='getDefaultXInfo',
+        write_permission=EDIT_ADVANCED_PERMISSION,
         read_permission=ModifyPortalContent,
         vocabulary=DisplayList( (
             ('HTTP_X_FORWARDED_FOR','HTTP_X_FORWARDED_FOR',),
@@ -294,6 +316,8 @@ formMailerAdapterSchema = FormAdapterSchema.copy() + Schema((
         schemata='headers',
         searchable=0,
         required=0,
+        default_method='getDefaultAddHdrs',
+        write_permission=EDIT_ADVANCED_PERMISSION,
         read_permission=ModifyPortalContent,
         widget=LinesWidget(
             label = 'Additional Headers',
@@ -307,20 +331,21 @@ formMailerAdapterSchema = FormAdapterSchema.copy() + Schema((
             ),
         ),
 ))
-    
+
 if gpg is not None:
     formMailerAdapterSchema = formMailerAdapterSchema + Schema((
         StringField('gpg_keyid',
             schemata='encryption',
             accessor='getGPGKeyId',
             mutator='setGPGKeyId',
+            write_permission=USE_ENCRYPTION_PERMISSION,
             read_permission=ModifyPortalContent,
             widget=StringWidget(
                 description = """
                     Give your key-id, e-mail address or
                     whatever works to match a public key from current keyring.
                     It will be used to encrypt the message body.
-                    Contact the site administrator if you need to 
+                    Contact the site administrator if you need to
                     install a new public key.
                     Note that you will probably wish to change your message
                     template to plain text if you're using encryption.
@@ -332,9 +357,90 @@ if gpg is not None:
                 i18n_domain='ploneformgen',
                 ),
             ),
-        ))    
-    
-finalizeATCTSchema(formMailerAdapterSchema, folderish=True, moveDiscussion=False)
+        ))
+
+
+formMailerAdapterSchema = formMailerAdapterSchema + Schema((
+    TALESString('subjectOverride',
+        schemata='overrides',
+        searchable=0,
+        required=0,
+        validators=('talesvalidator',),
+        default='',
+        write_permission=EDIT_TALES_PERMISSION,
+        read_permission=ModifyPortalContent,
+        isMetadata=True, # just to hide from base view
+        widget=StringWidget(label="Subject Expression",
+            description="""
+                A TALES expression that will be evaluated to override any value
+                otherwise entered for the e-mail subject header.
+                Leave empty if unneeded. Your expression should evaluate as a string.
+                PLEASE NOTE: errors in the evaluation of this expression will cause
+                an error on form display.
+            """,
+            size=70,
+            i18n_domain = "ploneformgen",
+            label_msgid = "label_subject_override_text",
+            description_msgid = "help_subject_override_text",
+        ),
+    ),
+    TALESString('senderOverride',
+        schemata='overrides',
+        searchable=0,
+        required=0,
+        validators=('talesvalidator',),
+        default='',
+        write_permission=EDIT_TALES_PERMISSION,
+        read_permission=ModifyPortalContent,
+        isMetadata=True, # just to hide from base view
+        widget=StringWidget(label="Sender Expression",
+            description="""
+                A TALES expression that will be evaluated to override the "From" header.
+                Leave empty if unneeded. Your expression should evaluate as a string.
+                PLEASE NOTE: errors in the evaluation of this expression will cause
+                an error on form display.
+            """,
+            size=70,
+            i18n_domain = "ploneformgen",
+            label_msgid = "label_sender_override_text",
+            description_msgid = "help_sender_override_text",
+        ),
+    ),
+    TALESString('recipientOverride',
+        schemata='overrides',
+        searchable=0,
+        required=0,
+        validators=('talesvalidator',),
+        default='',
+        write_permission=EDIT_TALES_PERMISSION,
+        read_permission=ModifyPortalContent,
+        isMetadata=True, # just to hide from base view
+        widget=StringWidget(label="Recipient Expression",
+            description="""
+                A TALES expression that will be evaluated to override any value
+                otherwise entered for the recipient e-mail address. You are strongly
+                cautioned against using unvalidated data from the request for this purpose.
+                Leave empty if unneeded. Your expression should evaluate as a string.
+                PLEASE NOTE: errors in the evaluation of this expression will cause
+                an error on form display.
+            """,
+            size=70,
+            i18n_domain = "ploneformgen",
+            label_msgid = "label_recipient_override_text",
+            description_msgid = "help_recipient_override_text",
+        ),
+    ),
+))
+
+if HAS_PLONE30:
+    # move headers schema items to template schema to keep the schema
+    # count <= 6
+    formMailerAdapterSchema['xinfo_headers'].schemata = 'template'
+    formMailerAdapterSchema['additional_headers'].schemata = 'template'
+
+formMailerAdapterSchema.moveField('execCondition', pos='bottom')
+if not HAS_PLONE30:
+    finalizeATCTSchema(formMailerAdapterSchema, folderish=True, moveDiscussion=False)
 
 
 class FormMailerAdapter(FormActionAdapter):
@@ -368,28 +474,92 @@ class FormMailerAdapter(FormActionAdapter):
         """
         e-mails data.
         """
-        
+
         self.send_form(fields, REQUEST)
 
 
+    security.declarePrivate('getMailBodyDefault')
+    def getMailBodyDefault(self):
+        """ Get default mail body from our tool """
+
+        fgt = getToolByName(self, 'formgen_tool')
+        return fgt.getDefaultMailTemplateBody()
+
+
+    security.declarePrivate('getMailBodyTypeDefault')
+    def getMailBodyTypeDefault(self):
+        """ Get default mail body type from our tool """
+
+        fgt = getToolByName(self, 'formgen_tool')
+        return fgt.getDefaultMailBodyType()
+
+
+    security.declarePrivate('getDefaultRecipient')
+    def getDefaultRecipient(self):
+        """ Get default mail recipient from our tool """
+
+        fgt = getToolByName(self, 'formgen_tool')
+        return fgt.getDefaultMailRecipient()
+
+
+    security.declarePrivate('getDefaultRecipientName')
+    def getDefaultRecipientName(self):
+        """ Get default mail recipient from our tool """
+
+        fgt = getToolByName(self, 'formgen_tool')
+        return fgt.getDefaultMailRecipientName()
+
+
+    security.declarePrivate('getDefaultCC')
+    def getDefaultCC(self):
+        """ Get default mail cc from our tool """
+
+        fgt = getToolByName(self, 'formgen_tool')
+        return fgt.getDefaultMailCC()
+
+
+    security.declarePrivate('getDefaultBCC')
+    def getDefaultBCC(self):
+        """ Get default mail bcc from our tool """
+
+        fgt = getToolByName(self, 'formgen_tool')
+        return fgt.getDefaultMailBCC()
+
+
+    security.declarePrivate('getDefaultXInfo')
+    def getDefaultXInfo(self):
+        """ Get default mail xinfo hdrs from our tool """
+
+        fgt = getToolByName(self, 'formgen_tool')
+        return fgt.getDefaultMailXInfo()
+
+
+    security.declarePrivate('getDefaultAddHdrs')
+    def getDefaultAddHdrs(self):
+        """ Get default mail add headers from our tool """
+
+        fgt = getToolByName(self, 'formgen_tool')
+        return fgt.getDefaultMailAddHdrs()
+
+
     security.declarePrivate('get_mail_text')
     def get_mail_text(self, fields, request, **kwargs):
         """Get header and body of e-mail as text (string)
         """
 
-        (headerinfo, additional_headers, 
+        (headerinfo, additional_headers,
          body) = self.get_header_body_tuple(fields, request, **kwargs)
 
-        mime_text = MIMEText(body, 
+        mime_text = MIMEText(body,
             _subtype=self.body_type or 'html', _charset=self._site_encoding())
 
         attachments = self.get_attachments(fields, request)
-        
-        if attachments:        
+
+        if attachments:
             outer = MIMEMultipart()
             outer.attach(mime_text)
         else:
-            outer = mime_text        
+            outer = mime_text
 
         # write header
         for key, value in headerinfo.items():
@@ -408,9 +578,9 @@ class FormMailerAdapter(FormActionAdapter):
 
             if ctype is None:
                 ctype = 'application/octet-stream'
-                 
+
             maintype, subtype = ctype.split('/', 1)
-             
+
             if maintype == 'text':
                 msg = MIMEText(content, _subtype=subtype)
             elif maintype == 'image':
@@ -422,7 +592,7 @@ class FormMailerAdapter(FormActionAdapter):
                 msg.set_payload(content)
                 # Encode the payload using Base64
                 Encoders.encode_base64(msg)
-                 
+
             # Set the filename parameter
             msg.add_header('Content-Disposition', 'attachment', filename=filename)
             outer.attach(msg)
@@ -435,12 +605,11 @@ class FormMailerAdapter(FormActionAdapter):
         """
 
         from ZPublisher.HTTPRequest import FileUpload
-        from OFS.content_types import guess_content_type
 
         attachments = []
-        
+
         for field in fields:
-            if field.meta_type == 'FormFileField':
+            if field.isFileField():
                 file = request.form.get('%s_file' % field.__name__, None)
                 if file and isinstance(file, FileUpload) and file.filename != '':
                     file.seek(0) # rewind
@@ -449,7 +618,7 @@ class FormMailerAdapter(FormActionAdapter):
                     mimetype, enc = guess_content_type(filename, data, None)
                     attachments.append((filename, mimetype, enc, data))
         return attachments
-                    
+
     security.declarePrivate('get_mail_body')
     def get_mail_body(self, fields, **kwargs):
         """Returns the mail-body with footer.
@@ -460,11 +629,18 @@ class FormMailerAdapter(FormActionAdapter):
         else:
             request = self.REQUEST
 
-        live_fields = [f.fgField for f in fields
-          if f.meta_type not in ('FormFileField','FormLabelField','FormRichLabelField')]
+        live_fields = [f for f in fields
+          if not (f.isLabel() or f.isFileField())]
+        bare_fields = [f.fgField for f in live_fields]
         bodyfield = self.getField('body_pt')
-        body = bodyfield.get(self, fields=live_fields)
-        
+        # pass both the bare_fields (fgFields only) and full fields.
+        # bare_fields for compatability with older templates,
+        # full fields to enable access to htmlValue
+        body = bodyfield.get(self, fields=bare_fields, wrappedFields=live_fields)
+
+        if isinstance(body, unicode):
+            body = body.encode(self.getCharset())
+
         keyid = getattr(self, 'gpg_keyid', None)
         encryption = gpg and keyid
 
@@ -484,9 +660,27 @@ class FormMailerAdapter(FormActionAdapter):
         if nlpos >= 0:
             line = line[:nlpos]
         return line
-        
+
+
+    security.declarePrivate('_destFormat')
+    def _destFormat(self, input):
+        """ Format destination (To) input.
+            Input may be a string or sequence of strings;
+            returns a well-formatted address field
+        """
+
+        if type(input) in StringTypes:
+            input = [s.strip().encode('utf-8') for s in input.split(',')]
+
+        filtered_input = [s for s in input if s]
+        if filtered_input:        
+            return "<%s>" % '>, <'.join( filtered_input )
+        else:
+            return ''
+
+
     security.declarePrivate('get_header_body_tuple')
-    def get_header_body_tuple(self, fields, request, 
+    def get_header_body_tuple(self, fields, request,
                               from_addr=None, to_addr=None,
                               subject=None, **kwargs):
         """Return header and body of e-mail as an 3-tuple:
@@ -504,52 +698,80 @@ class FormMailerAdapter(FormActionAdapter):
 #            request = self.REQUEST
 
         body = self.get_mail_body(fields, **kwargs)
-        
+
         # fields = self.fgFields()
 
+        # get Reply-To
         reply_addr = None
         if shasattr(self, 'replyto_field'):
             reply_addr = request.form.get(self.replyto_field, None)
 
-        nosubject = '(no subject)'                
-        subject = getattr(self, 'msg_subject', nosubject)
-        if getattr(self, 'subject_field', None):
-            subject = request.form.get(self.subject_field, subject)
+        # get subject header
+        nosubject = '(no subject)'
+        if shasattr(self, 'subjectOverride') and self.getRawSubjectOverride():
+            # subject has a TALES override
+            subject = self.getSubjectOverride().strip()
+        else:
+            subject = getattr(self, 'msg_subject', nosubject)
+            if getattr(self, 'subject_field', None):
+                subject = request.form.get(self.subject_field, subject)
+
+        # Get From address
+        if shasattr(self, 'senderOverride') and self.getRawSenderOverride():
+            from_addr = self.getSenderOverride().strip()
+        else:
+            pprops = getToolByName(self, 'portal_properties')
+            site_props = getToolByName(pprops, 'site_properties')
+            portal = getToolByName(self, 'portal_url').getPortalObject()
+            from_addr = site_props.getProperty('email_from_address') or \
+                        portal.getProperty('email_from_address')
+
+        # Get To address and full name
+        if shasattr(self, 'recipientOverride') and self.getRawRecipientOverride():
+            recip_email = self.getRecipientOverride()
+        else:
+            recip_email = None
+            if shasattr(self, 'to_field'):
+                recip_email = request.form.get(self.to_field, None)
+            if not recip_email:
+                recip_email = self.recipient_email
+        recip_email = self._destFormat( recip_email )
 
-        pprops = getToolByName(self, 'portal_properties')
-        site_props = getToolByName(pprops, 'site_properties')
-        portal = getToolByName(self, 'portal_url').getPortalObject()
-        from_addr = site_props.getProperty('email_from_address') or \
-                    portal.getProperty('email_from_address')
         recip_name = self.recipient_name.encode('utf-8')
 
-        recip_email = None
-        if shasattr(self, 'to_field'):
-            recip_email = request.form.get(self.to_field, None)
-        if not recip_email:
-            recip_email = self.recipient_email.encode('utf-8')
 
-        # if no to_addr and no recip_email specified : use owner adress if possible
+        # if no to_addr and no recip_email specified, use owner adress if possible.
+        # if not, fall back to portal email_from_address.
+        # if still no destination, raise an assertion exception.
         if not recip_email and not to_addr:
             ownerinfo = self.getOwner()
             ownerid=ownerinfo.getId()
             pms = getToolByName(self, 'portal_membership')
             userdest = pms.getMemberById(ownerid)
             toemail = userdest.getProperty('email', '')
-            assert toemail, 'You must specify a recipient e-mail address in the mail adapter.'
+            if not toemail:
+                portal = getToolByName(self, 'portal_url').getPortalObject()
+                toemail = portal.getProperty('email_from_address')                
+            assert toemail, """
+                    Unable to mail form input because no recipient address has been specified.
+                    Please check the recipient settings of the PloneFormGen "Mailer" within the
+                    current form folder.
+                """
             to = '%s <%s>' %(ownerid,toemail)
         else:
-            to = to_addr or '%s <%s>' % (recip_name, recip_email)
-
-        # transform subject into mail header encoded string
-        msgSubject = str(Header(subject,self._site_encoding()))
+            to = to_addr or '%s %s' % (recip_name, recip_email)
 
         headerinfo = OrderedDict()
-        headerinfo['To'] = to
-        headerinfo['From'] = from_addr
+
+        headerinfo['To'] = self.secure_header_line(to)
+        headerinfo['From'] = self.secure_header_line(from_addr)
         if reply_addr:
-            headerinfo['Reply-To'] = reply_addr
+            headerinfo['Reply-To'] = self.secure_header_line(reply_addr)
+
+        # transform subject into mail header encoded string
+        msgSubject = str(Header(self.secure_header_line(subject), self._site_encoding()))
         headerinfo['Subject'] = msgSubject
+
         headerinfo['MIME-Version'] = '1.0'
 
         # CC
@@ -565,10 +787,7 @@ class FormMailerAdapter(FormActionAdapter):
             headerinfo['Bcc'] = ', '.join(addrs)
 
         for key in getattr(self, 'xinfo_headers', []):
-            headerinfo['X-%s' % key] = request.get(key, 'MISSING')
-            
-        for key in headerinfo.keys(): 
-            headerinfo[key] = self.secure_header_line(headerinfo[key])
+            headerinfo['X-%s' % key] = self.secure_header_line(request.get(key, 'MISSING'))
 
         # return 3-Tuple
         return (headerinfo, self.additional_headers, body)
@@ -592,8 +811,8 @@ class FormMailerAdapter(FormActionAdapter):
 
     def fieldsDisplayList(self):
         """ returns display list of fields with simple values """
-        
-        return self.fgFieldsDisplayList(withNone=True, 
+
+        return self.fgFieldsDisplayList(withNone=True,
             objTypes=(
                 'FormSelectionField',
                 'FormStringField',
@@ -603,10 +822,11 @@ class FormMailerAdapter(FormActionAdapter):
 
     def selectFieldsDisplayList(self):
         """ returns display list of selection fields """
-        
-        return self.fgFieldsDisplayList(withNone=True, 
+
+        return self.fgFieldsDisplayList(withNone=True,
             objTypes=(
                 'FormSelectionField',
+                'FormMultiSelectionField',
                 )
             )
 
